/******************************************************************************
 * tipper_red.c
 * Federico Bareilles
 * 31/01/2003
 *
 *
 *****************************************************************************/
#define VERSION "0.1-c"

#define _POSIX_SOURCE 1         /* POSIX compliant source  */
//#define VERBOSE_MODE

#include <stdio.h>
#include <getopt.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "tipper_red.h"


char *program_name;

/* Flags: */
int verbose_flag      = 0;

static float spillover_sr6(float airmass)
{
  /*** SPILLOVER CORRECTION FACTOR:   ***/
  /*** Determined from SR6, July 1995 ***/
  /* ?????????????????????????????????? */
  return 1.0 - (0.0208 * airmass - 0.0164);
}

static float spillover_null(float airmas)
{
  return 1.0;
}

static float y_transform_null(float y)
{
  return y;
}

static float y_transform_log(float y)
{
  return log(y);
}

static float sigma_no_linear(struct tpoint *p)
{
  return sqrt(SQR(p->v_sky_sigma)+SQR(p->v_ref_sigma));
}
   
static float sigma_linear_log(struct tpoint *p)
{
  return sigma_no_linear(p)/(p->v_ref - GAINOFF * p->v_sky);
}



int main(int argc, char *argv[])
{

  char *cp; /* for options */
  //char line[160];
  static char *file_in;
  FILE *fp_in,*fp_out;
  int read_ang = MAX_ANG;
  int i;
  struct setup *s;

  int correction_flag    = 1;
  int reduction_flag     = 1;
  int spillover_flag     = 1;
  int nolinear_flag      = 0;
  int linear_matrix_flag = 0;
  int tempe_flag         = 0;
  int tempe_sigma_flag   = 0;

  int sec_offset = 0;                    /* for correction the coputer clock */

  int op;
  static struct option  const  longopts[] =
  {
    {"help", 0, 0, 101},
    {"verbose", 0, 0, 'v'},
    {"time", 1, 0, 't'},
    {"angles", 1, 0, 'a'},
    {"nocorrection", 0, 0, 102},
    {"nospillover", 0, 0, 's'},
    {"noreduction", 0, 0, 'n'},
    {"nolinear", 0, 0, 'l'},
    {"lmatrix", 0, 0, 'm'},
    {"tempe", 0, 0, 103},
    {"tsig", 0, 0, 104},
    {0,  0,  0,  0}
  };

  /* who is my name? */
  if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
  else program_name  = argv[0];
  opterr =  0;
  while  ((op = getopt_long(argc, argv, "vt:a:snlm",longopts, 0)) != EOF)
    switch( op ){
    case 'v':
      verbose_flag = 1;
      break;
    case 't':
      sec_offset=atoi(optarg);
      break;
    case 'a':
      read_ang=atoi(optarg);
      break;
    case 102:
      correction_flag = 0;
      break;
    case 's':
      spillover_flag = 0;
      break;
    case 'n':
      reduction_flag = 0;
      break;
    case 'l':
      nolinear_flag = 1;
      break;
    case 'm':
      linear_matrix_flag = 1;
      break;
    case 103:
      tempe_flag = 1;
      break;
    case 104:
      tempe_sigma_flag = 1;
      break;
    case 101:
      usage();
      help();
      exit(-1);
    default:
      usage();
      exit(-1);
    }
  if(tempe_flag){
    if(reduction_flag){
      reduction_flag = 0;
      PVERB("# WARNING: Reduction is ignored.\n");
    }
  }
  if(linear_matrix_flag && nolinear_flag){
    PVERB("Error: ambiguous flags, l and m impossible.\n\n");
    exit(1);
  }

  if(read_ang > MAX_ANG || read_ang < 3){
    PVERB("# ERROR: angles is 3 to %d\n",  MAX_ANG);
    read_ang = MAX_ANG;
  }
  if (optind < argc){
    file_in = argv[optind];
    if( strcmp(file_in,"-" ) == 0){
      file_in = (char *) malloc(sizeof(char)*8);
      strcpy(file_in,"stdin");
      fp_in = stdin;
    }else{
      if((fp_in=fopen(file_in,"r"))==NULL){
	PVERB("Error opening file %s data input.\n\n",file_in);
	exit(1);
      }
    }
    PVERB("# Data file input: %s\n",file_in);
    optind++;
  }else{
    PVERB("No input data file.\n");
    exit(-1);
  }
  if (optind < argc && strcmp(argv[optind],"-" ) ){
    if((fp_out=fopen(argv[optind],"w+"))==NULL){
      PVERB("Error opening file %s data output.\n\n",argv[optind]);
      exit(1);
    }
    PVERB("# Data file output: %s\n",argv[optind]);
    optind++;
  }else{
    fp_out=stdout;
    PVERB("# Data file output: stdout\n");
  }

  s = (struct setup*)malloc(sizeof(struct setup));
  s->file_in = file_in;
  s->read_ang = read_ang;
  s->sec_offset = sec_offset;

  /* Setup functions: */  
  if(spillover_flag)
    s->spillover=spillover_sr6;
  else{
    s->spillover=spillover_null;
    fprintf(fp_out,"# WARNING: No Spillover correction.\n");
  }
  if(  nolinear_flag || linear_matrix_flag){
    if(nolinear_flag){
      s->fit_metod=no_linear_fit;
    }else{
      s->fit_metod=linear_matrix_fit;
    }
    s->y_transform=y_transform_null;
    s->sigma_propagation=sigma_no_linear;
  }else{
    s->fit_metod=linear_fit;
    s->y_transform=y_transform_log;
    s->sigma_propagation=sigma_linear_log;
  }

  /* Raw data reading: */
  get_raw_data(fp_in, s);
  fclose(fp_in);
  s->tipper = set_instrument(s->data->unit); /* unit for first point */

  /* Corect voltages and no linear zener efect. */
  if(correction_flag)
    data_correction(s);
  else{ 
    PVERB("# WARNING: no correction voltages and linear zener efect.\n");
  }
  if( (i = data_check_airmass(s))){
    fprintf(fp_out,"# WARNING: %d obstacles detected.\n",i);
  }
  
  if(reduction_flag){
    data_reduction(s);
    if(! tempe_sigma_flag ){
      print_data(fp_out, s);
    }else{
      print_tsigma(fp_out, s);
    }
  }else{
    if(tempe_flag){
      print_tempe(fp_out, s);
    }else{
      print_data_no_red(fp_out, s);
    }
  }
  
  free_instrument(s->tipper);
  free_tdata(s->data);
  free(s);
  fclose(fp_out);

  return 0;
}
